from django.forms import ModelForm
from opcalendar.models import (
    Event,
    EventMember,
    EventCategory,
    EventHost,
    EventVisibility,
)
from django.forms.widgets import TextInput
from django import forms
from allianceauth.services.hooks import get_extension_logger

logger = get_extension_logger(__name__)


class EventForm(ModelForm):
    class Meta:
        model = Event
        # datetime-local is a HTML5 input type, format to make date time show on fields
        exclude = ["user", "eve_character", "created_date", "external"]

    def __init__(self, *args, **kwargs):
        super(EventForm, self).__init__(*args, **kwargs)

        self.fields["start_time"].input_formats = ("%Y-%m-%dT%H:%M",)
        self.fields["end_time"].input_formats = ("%Y-%m-%dT%H:%M",)
        self.fields["host"].queryset = EventHost.objects.filter(external=False)
        self.fields["event_visibility"].required = True
        self.fields["event_visibility"].queryset = EventVisibility.objects.filter(
            is_visible=True, is_active=True
        )
        try:
            self.initial["event_visibility"] = EventVisibility.objects.get(
                is_default=True
            )
        except Exception:
            logger.debug("Form defaults: No default visibility set")

        try:
            self.initial["host"] = EventHost.objects.get(is_default=True)
        except Exception:
            logger.debug("Form defaults: No default host set")


class EventEditForm(ModelForm):
    class Meta:
        model = Event
        # datetime-local is a HTML5 input type, format to make date time show on fields
        exclude = [
            "user",
            "eve_character",
            "created_date",
            "external",
            "repeat_event",
            "repeat_times",
        ]

    def __init__(self, *args, **kwargs):
        super(EventEditForm, self).__init__(*args, **kwargs)

        self.fields["start_time"].input_formats = ("%Y-%m-%dT%H:%M",)
        self.fields["end_time"].input_formats = ("%Y-%m-%dT%H:%M",)
        self.fields["host"].queryset = EventHost.objects.filter(external=False)
        self.fields["event_visibility"].required = True
        self.fields["event_visibility"].queryset = EventVisibility.objects.filter(
            is_visible=True, is_active=True
        )


class SignupForm(forms.Form):
    username = forms.CharField(
        widget=forms.TextInput(
            attrs={"class": "form-control", "placeholder": "Username"}
        )
    )
    password = forms.CharField(
        widget=forms.PasswordInput(
            attrs={"class": "form-control", "placeholder": "Password"}
        )
    )


class AddMemberForm(forms.ModelForm):
    class Meta:
        model = EventMember
        fields = ["character"]


class AddCategoryForm(forms.ModelForm):
    class Meta:
        model = EventCategory
        fields = "__all__"


class EventVisibilityAdminForm(forms.ModelForm):
    class Meta:
        model = EventVisibility
        fields = "__all__"
        widgets = {
            "color": TextInput(attrs={"type": "color"}),
        }


class EventCategoryAdminForm(forms.ModelForm):
    class Meta:
        model = EventCategory
        fields = "__all__"
        widgets = {
            "color": TextInput(attrs={"type": "color"}),
        }



import json
from datetime import datetime, timedelta
from typing import Tuple

import requests
from allianceauth.authentication.models import CharacterOwnership, State
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from allianceauth.services.hooks import get_extension_logger
from django.contrib.auth.models import Group, User
from django.db import models
from django.urls import reverse
from django.utils import timezone
from django.utils.html import strip_tags
from django.utils.translation import gettext_lazy as _
from esi.errors import TokenExpiredError, TokenInvalidError
from esi.models import Token

from .decorators import fetch_token_for_owner
from .providers import esi

logger = get_extension_logger(__name__)


class General(models.Model):
    """Meta model for app permissions"""

    class Meta:
        managed = False
        default_permissions = ()
        permissions = (
            (
                "basic_access",
                "Can access this app and see operations based on visibility rules",
            ),
            ("create_event", "Can create and edit events"),
            ("see_signups", "Can see all signups for event"),
            ("manage_event", "Can delete and manage other signups"),
            (
                "add_ingame_calendar_owner",
                "Can add ingame calendar feeds for their corporation",
            ),
        )


class WebHook(models.Model):
    """Discord Webhook for pings"""

    name = models.CharField(
        max_length=150,
        help_text=_("Name for this webhook"),
    )
    webhook_url = models.CharField(
        max_length=500,
        help_text=_("Webhook URL"),
    )
    enabled = models.BooleanField(default=True, help_text=_("Is the webhook enabled?"))

    def send_embed(self, embed):
        custom_headers = {"Content-Type": "application/json"}
        data = '{"embeds": [%s]}' % json.dumps(embed)
        r = requests.post(self.webhook_url, headers=custom_headers, data=data)
        r.raise_for_status()

    class Meta:
        verbose_name = "Webhook"
        verbose_name_plural = "Webhooks"

    def __str__(self):
        return "{}".format(self.name)


class EventVisibility(models.Model):
    name = models.CharField(
        max_length=150, null=False, help_text="Name for the visibility filter"
    )
    restricted_to_group = models.ManyToManyField(
        Group,
        blank=True,
        related_name="eventvisibility_require_groups",
        help_text=_(
            "The group(s) that will be able to see this event visibility type ..."
        ),
    )
    restricted_to_state = models.ManyToManyField(
        State,
        blank=True,
        related_name="eventvisibility_require_states",
        help_text=_(
            "The state(s) that will be able to see this event visibility type ..."
        ),
    )
    webhook = models.ForeignKey(
        WebHook,
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        help_text=_("Webhook to send over notifications about these fleet types"),
    )
    ignore_past_fleets = models.BooleanField(
        default=True,
        help_text=_("Should we ignore fleet signals that are in the past"),
    )
    color = models.CharField(
        max_length=7,
        default="",
        blank=True,
        help_text=_("Color to be displayed on calendar"),
    )
    include_in_feed = models.BooleanField(
        default=False,
        help_text=("Whether these events should be included in the ical feed."),
    )
    is_visible = models.BooleanField(
        default=True,
        help_text=(
            "Whether this visibility filter should be displayed on the event form. Disable for internal visibilities such as the NPSI import fleet visibilities."
        ),
    )
    is_default = models.BooleanField(
        default=False,
        help_text=(
            "Whether this visibility filter is used as the default value on the event form"
        ),
    )
    is_active = models.BooleanField(
        default=True,
        help_text=("Whether this visibility filter is active"),
    )

    def __str__(self) -> str:
        return str(self.name)

    class Meta:
        verbose_name = "Event Visibility Filter"
        verbose_name_plural = "Event Visibilities Filters"

    def save(self, *args, **kwargs):
        if self.is_default:
            # select all other is_default items
            qs = type(self).objects.filter(is_default=True)
            # except self (if self already exists)
            if self.pk:
                qs = qs.exclude(pk=self.pk)
            # and deactive them
            qs.update(is_default=False)

        super(EventVisibility, self).save(*args, **kwargs)

    @property
    def get_visibility_class(self):
        return f"{self.name.replace(' ', '-').lower()}"


class EventHost(models.Model):
    """Fleet Timer Create/Delete pings"""

    community = models.CharField(
        max_length=150, null=False, help_text="Name of the community"
    )
    logo_url = models.CharField(
        max_length=256, blank=True, help_text="Absolute URL for the community logo"
    )
    ingame_channel = models.CharField(
        max_length=150, blank=True, help_text="Ingame channel name"
    )
    ingame_mailing_list = models.CharField(
        max_length=150, blank=True, help_text="Ingame mailing list name"
    )
    fleet_comms = models.CharField(
        max_length=150,
        blank=True,
        help_text="Link or description for primary comms such as discord link",
    )
    fleet_doctrines = models.CharField(
        max_length=150, blank=True, help_text="Link or description to the doctrines"
    )
    website = models.CharField(max_length=150, blank=True, help_text="Website link URL")
    discord = models.CharField(max_length=150, blank=True, help_text="Discord link URL")
    twitch = models.CharField(max_length=150, blank=True, help_text="Twitch link URL")
    twitter = models.CharField(max_length=150, blank=True, help_text="Twitter link URL")
    youtube = models.CharField(max_length=150, blank=True, help_text="Youtube link URL")
    facebook = models.CharField(
        max_length=150, blank=True, help_text="Facebook link URL"
    )
    details = models.CharField(
        max_length=150, blank=True, help_text="Short description about the host."
    )
    is_default = models.BooleanField(
        default=False,
        help_text=("Whether this host is used as the default value on the event form"),
    )
    external = models.BooleanField(
        default=False,
        help_text=_(
            "External hosts are for NPSI API imports. Checking this box will hide the host in the manual event form."
        ),
    )

    def __str__(self):
        return str(self.community)

    class Meta:
        verbose_name = "Host"
        verbose_name_plural = "Hosts"

    def save(self, *args, **kwargs):
        if self.is_default:
            # select all other is_default items
            qs = type(self).objects.filter(is_default=True)
            # except self (if self already exists)
            if self.pk:
                qs = qs.exclude(pk=self.pk)
            # and deactive them
            qs.update(is_default=False)

        super(EventHost, self).save(*args, **kwargs)


class EventCategory(models.Model):
    name = models.CharField(
        max_length=150,
        help_text=_("Name for the category"),
    )
    ticker = models.CharField(
        max_length=10,
        help_text=_("Ticker for the category"),
    )
    color = models.CharField(
        max_length=7,
        default="",
        blank=True,
        help_text=_("Color to be displayed on calendar"),
    )
    description = models.TextField(
        blank=True,
        help_text="Prefilled description that will be added on default on the event description.",
    )

    class Meta:
        verbose_name = "Category"
        verbose_name_plural = "Categories"

    def __str__(self):
        return str(self.name)

    @property
    def get_category_class(self):
        return f"{self.name.replace(' ', '-').lower()}"


class EventImport(models.Model):
    """NPSI IMPORT OPTIONS"""

    SPECTRE_FLEET = "SF"
    EVE_UNIVERSITY = "EU"
    FUN_INC = "FI"
    FRIDAY_YARRRR = "FY"
    REDEMPTION_ROAD = "RR"
    CAS = "CA"
    FWAMING_DWAGONS = "FD"
    FREE_RANGE_CHIKUNS = "FR"
    EVE_LINKNET = "LN"

    IMPORT_SOURCES = [
        (EVE_LINKNET, _("EVE LinkNet")),
        (SPECTRE_FLEET, _("Spectre Fleet")),
        (EVE_UNIVERSITY, _("EVE University")),
        (FUN_INC, _("Fun Inc.")),
        (FRIDAY_YARRRR, _("FRIDAY YARRRR")),
        (REDEMPTION_ROAD, _("Redemption Road")),
        (CAS, _("CAS")),
        (FWAMING_DWAGONS, _("Fwaming Dwagons")),
        (FREE_RANGE_CHIKUNS, _("FREE RANGE CHIKUNS")),
    ]

    source = models.CharField(
        max_length=32,
        choices=IMPORT_SOURCES,
        help_text="The API source where you want to pull events from",
    )

    host = models.ForeignKey(
        EventHost,
        on_delete=models.CASCADE,
        default=1,
        help_text="The AA host that will be used for the pulled events",
    )
    operation_type = models.ForeignKey(
        EventCategory,
        on_delete=models.CASCADE,
        help_text="Operation type and ticker that will be assigned for the pulled fleets",
    )
    creator = models.ForeignKey(
        User,
        on_delete=models.CASCADE,
        default="1",
        help_text="User that has been used to create the fleet (most often the superuser who manages the plugin)",
    )
    eve_character = models.ForeignKey(
        EveCharacter,
        null=True,
        on_delete=models.SET_NULL,
        help_text="Event creator main character",
    )
    event_visibility = models.ForeignKey(
        EventVisibility,
        on_delete=models.CASCADE,
        null=True,
        help_text=_("Visibility filter that dictates who is able to see this event"),
    )

    def __str__(self):
        return str(self.source)

    class Meta:
        verbose_name = "NPSI Event Import"
        verbose_name_plural = "NPSI Event Imports"


class Event(models.Model):
    DAILY = "DD"
    WEEKLY = "WE"
    MONTHLY = "MM"
    YEARLY = "YY"

    REPEAT_INTERVAL = [
        (DAILY, _("Daily")),
        (WEEKLY, _("Weekly")),
        (MONTHLY, _("Monthly")),
        (YEARLY, _("Yearly")),
    ]

    operation_type = models.ForeignKey(
        EventCategory,
        null=True,
        on_delete=models.CASCADE,
        help_text=_("Event category type"),
    )
    title = models.CharField(
        max_length=200,
        help_text=_("Title for the event"),
    )
    host = models.ForeignKey(
        EventHost,
        on_delete=models.CASCADE,
        help_text=_("Host entity for the event"),
    )
    doctrine = models.CharField(
        max_length=254,
        help_text=_("Doctrine URL or name"),
    )
    formup_system = models.CharField(
        max_length=254,
        help_text=_("Location for formup"),
    )
    description = models.TextField(
        help_text=_("Description text for the operation"),
    )
    start_time = models.DateTimeField(
        help_text=_("Event start date and time"),
    )
    end_time = models.DateTimeField(
        help_text=_("Event end date and time"),
    )
    repeat_event = models.CharField(
        max_length=32,
        default=False,
        null=True,
        blank=True,
        choices=REPEAT_INTERVAL,
        help_text="Select if you want to repeat this event in the future",
    )
    repeat_times = models.IntegerField(
        default=False,
        null=True,
        blank=True,
        help_text="How many times do you want to repeat this event?",
    )
    fc = models.CharField(
        max_length=254,
        help_text=_("Fleet commander/manager for the event"),
    )
    event_visibility = models.ForeignKey(
        EventVisibility,
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        help_text=_("Visibility filter that dictates who is able to see this event"),
    )
    external = models.BooleanField(
        default=False,
        null=True,
        help_text=_("Is the event an external event over API"),
    )
    created_date = models.DateTimeField(
        default=timezone.now,
        help_text=_("When the event was created"),
    )
    eve_character = models.ForeignKey(
        EveCharacter,
        null=True,
        on_delete=models.SET_NULL,
        help_text=_("Character used to create the event"),
    )
    user = models.ForeignKey(
        User,
        on_delete=models.CASCADE,
        help_text=_("User who created the event"),
    )

    def duration(self):
        return self.end_time - self.start_time

    def __str__(self):
        return self.title

    def get_absolute_url(self):
        return reverse("opcalendar:event-detail", args=(self.id,))

    @property
    def get_visibility_class(self):
        if self.event_visibility:
            return f"{self.event_visibility.name.replace(' ', '-').lower()}"

    @property
    def get_event_styling(self):
        if self.event_visibility:
            return f".{self.event_visibility.name.replace(' ', '-').lower()}:before{{border-color: transparent {self.event_visibility.color} transparent transparent;border-style: solid;}} .{self.operation_type.name.replace(' ', '-').lower()} {{border-left: 6px solid {self.operation_type.color} !important;}}"

    @property
    def get_category_class(self):
        if self.operation_type:
            return f"{self.operation_type.name.replace(' ', '-').lower()}"

    @property
    def get_date_status(self):
        if datetime.now(timezone.utc) > self.start_time:
            return "past-event"
        else:
            return "future-event"

    @property
    def get_html_url(self):
        url = reverse("opcalendar:event-detail", args=(self.id,))
        return f"{url}"

    @property
    def get_html_title(self):
        return f'<span>{self.start_time.strftime("%H:%M")} - {self.end_time.strftime("%H:%M")} <i>{self.host.community}</i></span><span><b>{self.operation_type.ticker} {self.title}</b></span>'

    def user_can_edit(self, user: user) -> bool:
        """Checks if the given user can edit this timer. Returns True or False"""
        return user.has_perm("opcalendar.manage_event") or (
            self.user == user and user.has_perm("opcalendar.create_event")
        )


class Owner(models.Model):
    """A corporation that holds the calendars"""

    ERROR_NONE = 0
    ERROR_TOKEN_INVALID = 1
    ERROR_TOKEN_EXPIRED = 2
    ERROR_INSUFFICIENT_PERMISSIONS = 3
    ERROR_NO_CHARACTER = 4
    ERROR_ESI_UNAVAILABLE = 5
    ERROR_OPERATION_MODE_MISMATCH = 6
    ERROR_UNKNOWN = 99

    ERRORS_LIST = [
        (ERROR_NONE, "No error"),
        (ERROR_TOKEN_INVALID, "Invalid token"),
        (ERROR_TOKEN_EXPIRED, "Expired token"),
        (ERROR_INSUFFICIENT_PERMISSIONS, "Insufficient permissions"),
        (ERROR_NO_CHARACTER, "No character set for fetching data from ESI"),
        (ERROR_ESI_UNAVAILABLE, "ESI API is currently unavailable"),
        (
            ERROR_OPERATION_MODE_MISMATCH,
            "Operaton mode does not match with current setting",
        ),
        (ERROR_UNKNOWN, "Unknown error"),
    ]

    corporation = models.OneToOneField(
        EveCorporationInfo,
        default=None,
        null=True,
        blank=True,
        on_delete=models.CASCADE,
        help_text="Corporation owning the calendar",
        related_name="+",
    )
    character = models.ForeignKey(
        CharacterOwnership,
        on_delete=models.SET_DEFAULT,
        default=None,
        null=True,
        blank=True,
        help_text="Character used for syncing the calendar",
        related_name="+",
    )
    event_visibility = models.ForeignKey(
        EventVisibility,
        on_delete=models.CASCADE,
        null=True,
        blank=True,
        help_text=_("Visibility filter that dictates who is able to see this event"),
    )
    operation_type = models.ForeignKey(
        EventCategory,
        null=True,
        blank=True,
        on_delete=models.CASCADE,
        help_text=_(
            "Event category that will be assigned for all of the events from this owner."
        ),
    )
    is_active = models.BooleanField(
        default=True,
        help_text=("whether this owner is currently included in the sync process"),
    )

    class Meta:
        verbose_name = "Ingame Clanedar Owner"
        verbose_name_plural = "Ingame Calendar Owners"

    @fetch_token_for_owner(["esi-calendar.read_calendar_events.v1"])
    def update_events_esi(self, token):
        if self.is_active:
            # Get all current imported fleets in database
            event_ids_to_remove = list(
                IngameEvents.objects.filter(owner=self).values_list(
                    "event_id", flat=True
                )
            )
            logger.debug(
                "Ingame events currently in database: %s" % event_ids_to_remove
            )

            events = self._fetch_events()
            for event in events:
                character_id = self.character.character.character_id

                details = (
                    esi.client.Calendar.get_characters_character_id_calendar_event_id(
                        character_id=character_id,
                        event_id=event["event_id"],
                        token=token.valid_access_token(),
                    ).results()
                )

                end_date = event["event_date"] + timedelta(minutes=details["duration"])

                original = IngameEvents.objects.filter(
                    owner=self, event_id=event["event_id"]
                ).first()

                text = strip_tags(details["text"])

                try:
                    if original is not None:
                        logger.debug("Event: %s already in database" % event["title"])
                        event_ids_to_remove.remove(original.event_id)

                    else:
                        # Check if we already have the host
                        original_host = EventHost.objects.filter(
                            community=details["owner_name"]
                        ).first()

                        logger.debug("Got original host: {}".format(original_host))

                        if original_host is not None:
                            host = original_host
                        else:
                            host = EventHost.objects.create(
                                community=details["owner_name"],
                                external=True,
                            )

                        IngameEvents.objects.create(
                            event_id=event["event_id"],
                            owner=self,
                            text=text,
                            event_owner_id=details["owner_id"],
                            owner_type=details["owner_type"],
                            owner_name=details["owner_name"],
                            host=host,
                            importance=details["importance"],
                            duration=details["duration"],
                            event_start_date=event["event_date"],
                            event_end_date=end_date,
                            title=event["title"],
                        )
                        logger.debug("New event created: %s" % event["title"])
                except Exception as e:
                    logger.debug("Error adding new event: %s" % e)

            logger.debug("Removing all events that we did not get over API")
            IngameEvents.objects.filter(pk__in=event_ids_to_remove).delete()

            logger.debug(
                "All events fetched for %s" % self.character.character.character_name
            )

    @fetch_token_for_owner(["esi-calendar.read_calendar_events.v1"])
    def _fetch_events(self, token) -> list:
        character_id = self.character.character.character_id

        events = esi.client.Calendar.get_characters_character_id_calendar(
            character_id=character_id,
            token=token.valid_access_token(),
        ).results()

        return events

    def token(self, scopes=None) -> Tuple[Token, int]:
        """returns a valid Token for the owner"""
        token = None
        error = None

        # abort if character is not configured
        if self.character is None:
            logger.error("%s: No character configured to sync", self)
            error = self.ERROR_NO_CHARACTER

        # abort if character does not have sufficient permissions
        elif self.corporation and not self.character.user.has_perm(
            "opcalendar.add_ingame_calendar_owner"
        ):
            logger.error(
                "%s: This character does not have sufficient permission to sync corporation calendars",
                self,
            )
            error = self.ERROR_INSUFFICIENT_PERMISSIONS

        # abort if character does not have sufficient permissions
        elif not self.character.user.has_perm("opcalendar.add_ingame_calendar_owner"):
            logger.error(
                "%s: This character does not have sufficient permission to sync personal calendars",
                self,
            )
            error = self.ERROR_INSUFFICIENT_PERMISSIONS

        else:
            try:
                # get token
                token = (
                    Token.objects.filter(
                        user=self.character.user,
                        character_id=self.character.character.character_id,
                    )
                    .require_scopes(scopes)
                    .require_valid()
                    .first()
                )
            except TokenInvalidError:
                logger.error("%s: Invalid token for fetching calendars", self)
                error = self.ERROR_TOKEN_INVALID
            except TokenExpiredError:
                logger.error("%s: Token expired for fetching calendars", self)
                error = self.ERROR_TOKEN_EXPIRED
            else:
                if not token:
                    logger.error("%s: No token found with sufficient scopes", self)
                    error = self.ERROR_TOKEN_INVALID

        return token, error


class IngameEvents(models.Model):
    event_id = models.PositiveBigIntegerField(
        primary_key=True, help_text="The EVE ID of the event"
    )
    owner = models.ForeignKey(
        Owner,
        on_delete=models.CASCADE,
        help_text="Event holder",
    )
    event_start_date = models.DateTimeField()
    event_end_date = models.DateTimeField(blank=True, null=True)
    title = models.CharField(max_length=128)
    text = models.TextField()
    event_owner_id = models.IntegerField(null=True)
    owner_type = models.CharField(max_length=128)
    owner_name = models.CharField(max_length=128)
    host = models.ForeignKey(
        EventHost,
        on_delete=models.CASCADE,
        default=1,
        help_text=_("Host entity for the event"),
    )
    importance = models.CharField(max_length=128)
    duration = models.CharField(max_length=128)

    def __str__(self):
        return self.title

    class Meta:
        verbose_name = "Ingame Event"
        verbose_name_plural = "Ingame Events"

    def get_absolute_url(self):
        return reverse("opcalendar:ingame-event-detail", args=(self.event_id,))

    @property
    def get_date_status(self):
        if datetime.now(timezone.utc) > self.event_start_date:
            return "past-event"
        else:
            return "future-event"

    @property
    def get_visibility_class(self):
        if self.owner.event_visibility:
            return f"{self.owner.event_visibility.name.replace(' ', '-').lower()}"
        else:
            return "ingame-event"

    @property
    def get_event_styling(self):
        d = ""
        if self.owner.event_visibility:
            d += f".{self.owner.event_visibility.name.replace(' ', '-').lower()}:before{{border-color: transparent {self.owner.event_visibility.color} transparent transparent;border-style: solid;}}"
        if self.owner.operation_type:
            d += f".{self.owner.operation_type.name.replace(' ', '-').lower()} {{border-left: 6px solid {self.owner.operation_type.color} !important;}}"
        return d

    @property
    def get_category_class(self):
        if self.owner.operation_type:
            return f"{self.owner.operation_type.name.replace(' ', '-').lower()}"

    @property
    def get_html_url(self):
        url = reverse("opcalendar:ingame-event-detail", args=(self.event_id,))
        return f"{url}"

    @property
    def get_html_title(self):
        return f'<span>{self.event_start_date.strftime("%H:%M")} - {self.event_end_date.strftime("%H:%M")}<i> {self.owner_name}</i></span><span><b>{self.title}</b></span>'


class EventMember(models.Model):
    event = models.ForeignKey(Event, on_delete=models.CASCADE)
    character = models.ForeignKey(
        EveCharacter,
        null=True,
        on_delete=models.SET_NULL,
        help_text="Event creator main character",
    )

    class Meta:
        unique_together = ["event", "character"]



# Generated by Django 3.1.2 on 2020-11-04 11:14

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("opcalendar", "0002_auto_20201104_1054"),
    ]

    operations = [
        migrations.CreateModel(
            name="EventHost",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("community", models.CharField(max_length=150)),
            ],
            options={
                "verbose_name": "Host",
                "verbose_name_plural": "Hosts",
            },
        ),
    ]



import datetime
from django.utils import timezone


class time_helpers:
    @staticmethod
    def convert_timedelta(duration):
        # days = duration.days
        seconds = duration.seconds

        hours = seconds // 3600
        minutes = (seconds % 3600) // 60
        seconds = seconds % 60

        return hours, minutes, seconds

    @staticmethod
    def format_timedelta(td):
        hours, minutes, seconds = time_helpers.convert_timedelta(td)
        return "%d Days, %d Hours, %d Min" % (td.days, round(hours), round(minutes))

    @staticmethod
    def get_time_until(dt):
        """Return D / H / M Until DateTime"""
        return time_helpers.format_timedelta(
            dt.replace(tzinfo=timezone.utc) - datetime.datetime.now(timezone.utc)
        )



# Generated by Django 3.2.10 on 2022-01-20 11:58

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("opcalendar", "0025_auto_20220115_1101"),
    ]

    operations = [
        migrations.AlterField(
            model_name="event",
            name="repeat_event",
            field=models.CharField(
                blank=True,
                choices=[
                    ("DD", "Daily"),
                    ("WE", "Weekly"),
                    ("MM", "Monthly"),
                    ("YY", "Yearly"),
                ],
                default=False,
                help_text="Select if you want to repeat this event in the future",
                max_length=32,
                null=True,
            ),
        ),
    ]



# Generated by Django 3.1.2 on 2021-01-19 15:45

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("eveonline", "0012_index_additions"),
        ("authentication", "0017_remove_fleetup_permission"),
        ("opcalendar", "0015_eventimport_eve_character"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="general",
            options={
                "default_permissions": (),
                "managed": False,
                "permissions": (
                    ("basic_access", "Can access this app"),
                    ("view_public", "Can see public events"),
                    ("view_member", "Can see member events"),
                    ("view_ingame", "Can see events from ingame calendar"),
                    ("create_event", "Can create and edit events"),
                    ("manage_event", "Can delete and manage signups"),
                    (
                        "add_ingame_calendar_owner",
                        "Can add ingame calendar feeds for their corporation",
                    ),
                ),
            },
        ),
        migrations.CreateModel(
            name="Owner",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "is_active",
                    models.BooleanField(
                        default=True,
                        help_text="whether this owner is currently included in the sync process",
                    ),
                ),
                (
                    "character",
                    models.ForeignKey(
                        blank=True,
                        default=None,
                        help_text="character used for syncing blueprints",
                        null=True,
                        on_delete=django.db.models.deletion.SET_DEFAULT,
                        related_name="+",
                        to="authentication.characterownership",
                    ),
                ),
                (
                    "corporation",
                    models.OneToOneField(
                        blank=True,
                        default=None,
                        help_text="Corporation owning blueprints, if this is a 'corporate' owner",
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="eveonline.evecorporationinfo",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="IngameEvents",
            fields=[
                (
                    "event_id",
                    models.PositiveBigIntegerField(
                        help_text="The EVE ID of the event",
                        primary_key=True,
                        serialize=False,
                    ),
                ),
                ("event_start_date", models.DateTimeField()),
                ("event_end_date", models.DateTimeField(blank=True, null=True)),
                ("title", models.CharField(max_length=128)),
                ("text", models.TextField()),
                ("owner_type", models.CharField(max_length=128)),
                ("owner_name", models.CharField(max_length=128)),
                ("importance", models.CharField(max_length=128)),
                ("duration", models.CharField(max_length=128)),
                (
                    "owner",
                    models.ForeignKey(
                        help_text="Event holder",
                        on_delete=django.db.models.deletion.CASCADE,
                        to="opcalendar.owner",
                    ),
                ),
            ],
        ),
    ]



# Generated by Django 3.1.2 on 2021-01-12 09:34

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("opcalendar", "0011_eventimport"),
    ]

    operations = [
        migrations.AlterField(
            model_name="eventimport",
            name="source",
            field=models.CharField(
                choices=[
                    ("Spectre Fleet", "Spectre Fleet"),
                    ("EVE University", "EVE University"),
                ],
                max_length=32,
            ),
        ),
    ]



from django.conf import settings
from .utils import clean_setting
import re


def get_site_url():  # regex sso url
    regex = r"^(.+)\/s.+"
    matches = re.finditer(regex, settings.ESI_SSO_CALLBACK_URL, re.MULTILINE)
    url = "http://"

    for m in matches:
        url = m.groups()[0]  # first match

    return url


# Hard timeout for tasks in seconds to reduce task accumulation during outages
OPCALENDAR_TASKS_TIME_LIMIT = clean_setting("OPCALENDAR_TASKS_TIME_LIMIT", 7200)

# whether admins will get notifications about import events
OPCALENDAR_ADMIN_NOTIFICATIONS_ENABLED = clean_setting(
    "OPCALENDAR_ADMIN_NOTIFICATIONS_ENABLED", True
)

# whether we should send out discord notifications for imported fleets
OPCALENDAR_NOTIFY_IMPORTS = clean_setting("OPCALENDAR_NOTIFY_IMPORTS", True)

# whether we should inculde timers from the structuretimers plugin in the calendar
OPCALENDAR_DISPLAY_STRUCTURETIMERS = clean_setting(
    "OPCALENDAR_DISPLAY_STRUCTURETIMERS", True
)

# whether we should inculde extractions from the moonmining plugin in the calendar
OPCALENDAR_DISPLAY_MOONMINING = clean_setting("OPCALENDAR_DISPLAY_MOONMINING", True)

# If we want to display the moon mining tags in the opcalendar events
OPCALENDAR_DISPLAY_MOONMINING_TAGS = clean_setting(
    "OPCALENDAR_DISPLAY_MOONMINING_TAGS", True
)

# Display moon mining arrival time on opcalendar. If set to False will display auto fracture time instead
OPCALENDAR_DISPLAY_MOONMINING_ARRIVAL_TIME = clean_setting(
    "OPCALENDAR_DISPLAY_MOONMINING_ARRIVAL_TIME", True
)

# whether we display external hosts in the discord ops command filters
OPCALENDAR_DISCORD_OPS_DISPLAY_EXTERNAL = clean_setting(
    "OPCALENDAR_DISCORD_OPS_DISPLAY_EXTERNAL", False
)

OPCALENDAR_EVE_UNI_URL = "https://portal.eveuniversity.org/api/getcalendar"
OPCALENDAR_SPECTRE_URL = "https://www.spectre-fleet.space/engagement/events/rss"
OPCALENDAR_FUNINC_URL = "https://calendar.google.com/calendar/ical/og3uh76l8ul3dfgbie03fbbgs8%40group.calendar.google.com/private-f466889b44741fd7249e99e21ac171ff/basic.ics"
OPCALENDAR_FRIDAY_YARRRR_URL = "https://calendar.google.com/calendar/ical/vl43scrg7olk01fv7g79hsbe74%40group.calendar.google.com/public/basic.ics"
OPCALENDAR_REDEMPTION_ROAD_URL = "https://calendar.google.com/calendar/ical/5o3gpum6ek2irk12f0hnhfdtrs%40group.calendar.google.com/public/basic.ics"
OPCALENDAR_CAS_URL = "https://calendar.google.com/calendar/ical/0sqru3js6pb1p71e7n1ko91rqs%40group.calendar.google.com/public/basic.ics"
OPCALENDAR_FWAMING_DWAGONS_URL = "https://calendar.google.com/calendar/ical/l0mnjo7ormaq9gomap0cke4kqk%40group.calendar.google.com/public/basic.ics"
OPCALENDAR_FREE_RANGE_CHIKUNS_URL = "https://calendar.google.com/calendar/ical/2nabdlgsebhsv29qmhtjgd0u9k%40group.calendar.google.com/public/basic.ics"
OPCALENDAR_EVE_LINKNET_URL = "https://auth.eve-linknet.com/opcalendar/feed.ics"


def structuretimers_active():
    return "structuretimers" in settings.INSTALLED_APPS


def moonmining_active():
    return "moonmining" in settings.INSTALLED_APPS


# Use a small helper to check if AA-Discordbot is installs
def discord_bot_active():
    return "aadiscordbot" in settings.INSTALLED_APPS



import json
from pathlib import Path

from bravado.exception import HTTPNotFound
from django.utils.dateparse import parse_datetime
from ics import Calendar, Event

from ..utils import BravadoOperationStub, BravadoResponseStub

COLOR_PURPLE = "#8e44ad"


def _load_testdata() -> dict:
    testdata_path = Path(__file__).parent / "testdata.json"
    with testdata_path.open("r", encoding="utf-8") as fp:
        return json.load(fp)


_testdata = _load_testdata()


def generate_ical_string(key: str) -> str:
    """generates iCalendar string from testdata and returns it"""
    c = Calendar()
    for row in _testdata["iCalendar"].get(key, []):
        c.events.add(
            Event(
                name=row["name"],
                begin=row["begin"],
                end=row["end"],
                description=row["description"],
            )
        )
    return c.serialize()


class FeedsStub:
    """Generates feeds from testdata compatible with feedparser"""

    class FeedEntryStub:
        class AuthorDetail:
            def __init__(self, author_detail) -> None:
                self.name = author_detail.get("name", "") if author_detail else ""

        def __init__(self, entry) -> None:
            self.author_detail = self.AuthorDetail(entry.get("author_detail"))
            self.title = entry.get("title", "")
            self.published = entry.get("published", "")
            self.description = entry.get("description", "")

    def __init__(self, feed) -> None:
        self.entries = (
            [self.FeedEntryStub(row) for row in feed["entries"]] if feed else []
        )


def feedparser_parse(url) -> list:
    return FeedsStub(_testdata["feeds"].get(url))


def esi_get_characters_character_id_calendar(character_id, token):
    try:
        data = _testdata["esi"]["get_characters_character_id_calendar"][
            str(character_id)
        ]
        return BravadoOperationStub([_convert_rows(obj) for obj in data])
    except KeyError:
        raise HTTPNotFound(
            BravadoResponseStub(
                status_code=404, text=f"no data for character_id {character_id}"
            )
        )


def esi_get_characters_character_id_calendar_event_id(character_id, event_id, token):
    try:
        obj = _testdata["esi"]["get_characters_character_id_calendar_event_id"][
            str(character_id)
        ][str(event_id)]
        return BravadoOperationStub(_convert_rows(obj))
    except KeyError:
        raise HTTPNotFound(
            BravadoResponseStub(
                status_code=404,
                text=(f"no data for character_id {character_id}, event_id {event_id}"),
            )
        )


def _convert_rows(dct) -> dict:
    for key, value in dct.items():
        if isinstance(value, str):
            my_dt = parse_datetime(value)
            if my_dt:
                dct[key] = my_dt
    return dct



from datetime import datetime
import re

from bravado.exception import HTTPBadGateway, HTTPGatewayTimeout, HTTPServiceUnavailable
from celery import shared_task
import feedparser
from ics import Calendar
import requests
import pytz
from django.utils.html import strip_tags
from allianceauth.services.hooks import get_extension_logger
from allianceauth.services.tasks import QueueOnce

from .app_settings import (
    OPCALENDAR_EVE_UNI_URL,
    OPCALENDAR_SPECTRE_URL,
    OPCALENDAR_FUNINC_URL,
    OPCALENDAR_FRIDAY_YARRRR_URL,
    OPCALENDAR_REDEMPTION_ROAD_URL,
    OPCALENDAR_CAS_URL,
    OPCALENDAR_FWAMING_DWAGONS_URL,
    OPCALENDAR_FREE_RANGE_CHIKUNS_URL,
    OPCALENDAR_EVE_LINKNET_URL,
)
from .app_settings import OPCALENDAR_TASKS_TIME_LIMIT
from .models import Event, EventImport, Owner


DEFAULT_TASK_PRIORITY = 6

logger = get_extension_logger(__name__)

# Create your tasks here
TASK_DEFAULT_KWARGS = {
    "time_limit": OPCALENDAR_TASKS_TIME_LIMIT,
}

TASK_ESI_KWARGS = {
    **TASK_DEFAULT_KWARGS,
    **{
        "bind": True,
        "autoretry_for": (
            OSError,
            HTTPBadGateway,
            HTTPGatewayTimeout,
            HTTPServiceUnavailable,
        ),
        "retry_kwargs": {"max_retries": 3},
        "retry_backoff": 30,
    },
}


@shared_task
def import_all_npsi_fleets() -> bool:
    """Imports all NPSI fleets from their respective APIs"""

    # Get all current imported fleets in database
    event_ids_to_remove = set(
        Event.objects.filter(external=True).values_list("id", flat=True)
    )

    logger.debug("External events in database: {}".format(event_ids_to_remove))

    # Get all import feeds
    feeds = EventImport.objects.all()

    feed_errors = []
    map_ical_imports = {
        EventImport.FRIDAY_YARRRR: OPCALENDAR_FRIDAY_YARRRR_URL,
        EventImport.REDEMPTION_ROAD: OPCALENDAR_REDEMPTION_ROAD_URL,
        EventImport.CAS: OPCALENDAR_CAS_URL,
        EventImport.FREE_RANGE_CHIKUNS: OPCALENDAR_FREE_RANGE_CHIKUNS_URL,
        EventImport.EVE_LINKNET: OPCALENDAR_EVE_LINKNET_URL,
        EventImport.FWAMING_DWAGONS: OPCALENDAR_FWAMING_DWAGONS_URL,
    }

    # Check for active NPSI feeds
    for feed in feeds:
        # If Spectre Fleet is active
        if feed.source == EventImport.SPECTRE_FLEET:
            has_error = _import_spectre_fleet(feed, event_ids_to_remove)
            feed_errors.append(has_error)

        # Check for FUN Inc fleets
        if feed.source == EventImport.FUN_INC:
            has_error = _import_fun_inc(feed, event_ids_to_remove)
            feed_errors.append(has_error)

        # Check for EVE Uni events
        if feed.source == EventImport.EVE_UNIVERSITY:
            has_error = _import_eve_uni(feed, event_ids_to_remove)
            feed_errors.append(has_error)

        # Check for events from ical feeds
        for source, url in map_ical_imports.items():
            if feed.source == source:
                has_error = _import_ical(feed, event_ids_to_remove, url)
                feed_errors.append(has_error)

    logger.debug("Checking for NPSI fleets to be removed.")

    successful_imports = feed_errors.count(False)
    if successful_imports:
        logger.info(
            "Successfully imported %d / %d NPSI feeds.",
            successful_imports,
            len(feed_errors),
        )

    if any(feed_errors):
        logger.error("Errors in feeds, not cleaning up operations on this run")
        return False

    if not event_ids_to_remove:
        logger.debug("No NPSI fleets to be removed.")
    else:
        logger.debug("Removed unseen NPSI fleets")
        # Remove all events we did not see from API
        Event.objects.filter(pk__in=event_ids_to_remove).delete()
    return True


def _import_spectre_fleet(feed, event_ids_to_remove):
    logger.debug(
        "%s: import feed active. Pulling events from %s",
        feed,
        OPCALENDAR_SPECTRE_URL,
    )

    try:
        # Get spectre fleets from their RSS feed
        d = feedparser.parse(OPCALENDAR_SPECTRE_URL)

        # Process each fleet entry
        for entry in d.entries:
            # Look for SF fleets only
            if entry.author_detail.name == "Spectre Fleet":
                # Only active fleets
                if "[RESERVED]" not in entry.title:
                    logger.debug("%s: Import even found: %s", feed, entry.title)

                    # Format datetimes
                    date_object = datetime.strptime(
                        entry.published, "%a, %d %b %Y %H:%M:%S %z"
                    )
                    date_object.strftime("%Y-%m-%dT%H:%M")

                    # Check if we already have the event stored
                    original = Event.objects.filter(
                        start_time=date_object, title=entry.title
                    ).first()

                    # If we get the event from API it should not be removed
                    if original:
                        logger.debug(
                            "%s: Event: %s already in database, skipping",
                            feed,
                            entry.title,
                        )

                        # Remove the found fleet from the to be removed list
                        event_ids_to_remove.discard(original.id)

                    else:
                        # Save new fleet to database
                        Event.objects.create(
                            operation_type=feed.operation_type,
                            title=entry.title,
                            host=feed.host,
                            doctrine="see details",
                            formup_system=feed.source,
                            description=strip_tags(entry.description),
                            start_time=date_object,
                            end_time=date_object,
                            fc=feed.source,
                            external=True,
                            user=feed.creator,
                            event_visibility=feed.event_visibility,
                            eve_character=feed.eve_character,
                        )

                        logger.debug(
                            "%s: Saved new event in database: %s",
                            feed,
                            entry.title,
                        )

    except Exception:
        logger.error("%s: Error in fetching fleets", feed, exc_info=True)
        return True

    return False


def _import_fun_inc(feed, event_ids_to_remove):
    logger.debug(
        "%s: import feed active. Pulling events from %s",
        feed,
        OPCALENDAR_FUNINC_URL,
    )

    try:
        # Get FUN Inc fleets from google ical
        r = requests.get(OPCALENDAR_FUNINC_URL)
        c = Calendar(r.text)

        # Parse each entry we got
        for entry in c.events:
            # Format datetime
            start_date = datetime.utcfromtimestamp(entry.begin.float_timestamp).replace(
                tzinfo=pytz.utc
            )
            end_date = datetime.utcfromtimestamp(entry.end.float_timestamp).replace(
                tzinfo=pytz.utc
            )
            title = entry.name

            logger.debug("%s: Import even found: %s", feed, title)

            # Check if we already have the event stored
            original = Event.objects.filter(start_time=start_date, title=title).first()

            # If we get the event from API it should not be removed
            if original:
                logger.debug("%s: Event: %s already in database, skipping", feed, title)

                # Remove the found fleet from the to be removed list
                event_ids_to_remove.discard(original.id)

            else:
                # Save new fleet to database
                event = Event(
                    operation_type=feed.operation_type,
                    title=title,
                    host=feed.host,
                    doctrine="see details",
                    formup_system=feed.source,
                    description=strip_tags(entry.description),
                    start_time=start_date,
                    end_time=end_date,
                    fc=feed.source,
                    external=True,
                    user=feed.creator,
                    event_visibility=feed.event_visibility,
                    eve_character=feed.eve_character,
                )

                logger.debug("%s: Saved new EVE UNI event in database: %s", feed, title)

                event.save()

    except Exception:
        logger.error("%s: Error in fetching fleets", feed, exc_info=True)
        return True

    return False


def _import_eve_uni(feed, event_ids_to_remove):
    logger.debug(
        "%s: import feed active. Pulling events from %s",
        feed,
        OPCALENDAR_EVE_UNI_URL,
    )

    try:
        # Get EVE Uni events from their API feed (ical)
        r = requests.get(OPCALENDAR_EVE_UNI_URL)
        c = Calendar(r.text)
        for entry in c.events:
            # Filter only class events as they are the only public events in eveuni
            if "class" in entry.name.lower():
                # Format datetime
                start_date = datetime.utcfromtimestamp(
                    entry.begin.float_timestamp
                ).replace(tzinfo=pytz.utc)
                end_date = datetime.utcfromtimestamp(entry.end.float_timestamp).replace(
                    tzinfo=pytz.utc
                )
                title = re.sub(r"[\(\[].*?[\)\]]", "", entry.name)

                logger.debug("%s: Import even found: %s", feed, title)

                # Check if we already have the event stored
                original = Event.objects.filter(
                    start_time=start_date, title=title
                ).first()

                # If we get the event from API it should not be removed
                if original:
                    logger.debug(
                        "%s: Event: %s already in database, skipping", feed, title
                    )

                    # Remove the found fleet from the to be removed list
                    event_ids_to_remove.discard(original.id)

                else:
                    # Save new event to database
                    event = Event(
                        operation_type=feed.operation_type,
                        title=title,
                        host=feed.host,
                        doctrine="see details",
                        formup_system=feed.source,
                        description=strip_tags(entry.description.replace("<br>", "\n")),
                        start_time=start_date,
                        end_time=end_date,
                        fc=feed.source,
                        external=True,
                        user=feed.creator,
                        event_visibility=feed.event_visibility,
                        eve_character=feed.eve_character,
                    )

                    logger.debug(
                        "%s: Saved new EVE UNI event in database: %s",
                        feed,
                        title,
                    )
                    event.save()

    except Exception:
        logger.error("%s: Error in fetching fleets", feed, exc_info=True)
        return True

    return False


def _import_ical(feed, event_ids_to_remove, url):
    logger.debug(
        "%s: import feed active. Pulling events from %s",
        feed,
        url,
    )

    try:
        # Get EVE Uni events from their API feed (ical)
        r = requests.get(url)
        c = Calendar(r.text)
        for entry in c.events:
            # Format datetime
            start_date = datetime.utcfromtimestamp(entry.begin.float_timestamp).replace(
                tzinfo=pytz.utc
            )
            end_date = datetime.utcfromtimestamp(entry.end.float_timestamp).replace(
                tzinfo=pytz.utc
            )
            title = re.sub(r"[\(\[].*?[\)\]]", "", entry.name)

            logger.debug("%s: Import even found: %s", feed, title)

            # Check if we already have the event stored
            original = Event.objects.filter(start_time=start_date, title=title).first()

            # If we get the event from API it should not be removed
            if original:
                logger.debug("%s: Event: %s already in database, skipping", feed, title)

                # Remove the found fleet from the to be removed list
                event_ids_to_remove.discard(original.id)

            else:
                # Save new event to database
                event = Event(
                    operation_type=feed.operation_type,
                    title=title,
                    host=feed.host,
                    formup_system=entry.location,
                    description=strip_tags(entry.description.replace("<br>", "\n")),
                    start_time=start_date,
                    end_time=end_date,
                    external=True,
                    user=feed.creator,
                    event_visibility=feed.event_visibility,
                    eve_character=feed.eve_character,
                )

                logger.debug(
                    "%s: Saved new EVE UNI event in database: %s",
                    feed,
                    title,
                )
                event.save()

    except Exception:
        logger.error("%s: Error in fetching fleets", feed, exc_info=True)
        return True

    return False


@shared_task(
    **{
        **TASK_ESI_KWARGS,
        **{
            "base": QueueOnce,
            "once": {"keys": ["owner_pk"], "graceful": True},
            "max_retries": None,
        },
    }
)
def update_events_for_owner(self, owner_pk):
    """fetches all calendars for owner from ESI"""

    return _get_owner(owner_pk).update_events_esi()


@shared_task(**TASK_DEFAULT_KWARGS)
def update_all_ingame_events():
    for owner in Owner.objects.all():
        update_events_for_owner.apply_async(
            kwargs={"owner_pk": owner.pk},
            priority=DEFAULT_TASK_PRIORITY,
        )


def _get_owner(owner_pk: int) -> Owner:
    """returns the owner or raises exception"""
    try:
        owner = Owner.objects.get(pk=owner_pk)
    except Owner.DoesNotExist:
        raise Owner.DoesNotExist(
            "Requested owner with pk {} does not exist".format(owner_pk)
        )
    return owner


# @shared_task
# def add(x, y):
#     return x + y



# Generated by Django 3.1.2 on 2021-01-13 09:15

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("eveonline", "0012_index_additions"),
        ("opcalendar", "0014_auto_20210113_0836"),
    ]

    operations = [
        migrations.AddField(
            model_name="eventimport",
            name="eve_character",
            field=models.ForeignKey(
                help_text="Event creator main character",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="eveonline.evecharacter",
            ),
        ),
    ]



from django.contrib import admin
from django.utils.safestring import mark_safe
from opcalendar.models import (
    Event,
    EventCategory,
    WebHook,
    EventHost,
    EventImport,
    Owner,
    IngameEvents,
    EventVisibility,
)
from .forms import EventVisibilityAdminForm, EventCategoryAdminForm


def custom_filter(title):
    """
    custom filter for model properties
    :param title:
    :return:
    """

    class Wrapper(admin.FieldListFilter):
        """
        custom_filter :: wrapper
        """

        def __new__(cls, *args, **kwargs):
            instance = admin.FieldListFilter.create(*args, **kwargs)
            instance.title = title

            return instance

    return Wrapper


class WebHookAdmin(admin.ModelAdmin):
    list_display = ("name", "enabled")


admin.site.register(WebHook, WebHookAdmin)


@admin.register(EventCategory)
class EventCategoryAdmin(admin.ModelAdmin):
    form = EventCategoryAdminForm
    list_display = (
        "name",
        "ticker",
        "_color",
    )

    def _color(self, obj):
        html = (
            f'<input type="color" value="{obj.color}" disabled>' if obj.color else "-"
        )
        return mark_safe(html)


class EventHostAdmin(admin.ModelAdmin):
    model = EventHost

    list_display = (
        "community",
        "is_default",
        "external",
    )


admin.site.register(EventHost, EventHostAdmin)


@admin.register(EventImport)
class EventImportAdmin(admin.ModelAdmin):
    model = EventImport

    list_display = (
        "source",
        "host",
        "event_visibility",
        "operation_type",
    )


@admin.register(EventVisibility)
class EventVisibilityAdmin(admin.ModelAdmin):
    form = EventVisibilityAdminForm

    list_display = (
        "name",
        "_restricted_to_group",
        "_restricted_to_state",
        "_color",
        "is_default",
        "is_visible",
        "is_active",
    )
    filter_horizontal = ("restricted_to_group",)
    ordering = ("name",)

    list_filter = (
        ("is_active", custom_filter(title="active")),
        ("restricted_to_group", custom_filter(title="restriction")),
    )

    def _color(self, obj):
        html = (
            f'<input type="color" value="{obj.color}" disabled>' if obj.color else "-"
        )
        return mark_safe(html)

    @classmethod
    def _name(cls, obj):
        return obj.name

    _name.short_description = "Visibility"
    _name.admin_order_field = "name"

    @classmethod
    def _restricted_to_group(cls, obj):
        names = [x.name for x in obj.restricted_to_group.all().order_by("name")]

        if names:
            return ", ".join(names)
        else:
            return None

    _restricted_to_group.short_description = "Restricted to"
    _restricted_to_group.admin_order_field = "restricted_to_group__name"

    @classmethod
    def _restricted_to_state(cls, obj):
        names = [x.name for x in obj.restricted_to_state.all().order_by("name")]

        if names:
            return ", ".join(names)
        else:
            return None

    _restricted_to_state.short_description = "Restricted to"
    _restricted_to_state.admin_order_field = "restricted_to_state__name"


@admin.register(Owner)
class OwnerAdmin(admin.ModelAdmin):
    model = Owner

    list_display = (
        "character",
        "event_visibility",
        "operation_type",
        "is_active",
    )


@admin.register(IngameEvents)
class IngameEventsAdmin(admin.ModelAdmin):
    model = IngameEvents

    list_display = (
        "title",
        "owner_type",
        "owner_name",
    )


@admin.register(Event)
class EventAdmin(admin.ModelAdmin):
    model = Event

    list_display = (
        "title",
        "host",
        "operation_type",
        "event_visibility",
        "external",
    )



# Generated by Django 3.1.2 on 2020-11-04 10:54

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("opcalendar", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="eventcategory",
            name="color",
            field=models.CharField(
                choices=[
                    ("green", "Green"),
                    ("red", "Red"),
                    ("orange", "Orange"),
                    ("blue", "Blue"),
                    ("grey", "Grey"),
                    ("yellow", "Yellow"),
                    ("purple", "Purple"),
                ],
                default="green",
                max_length=6,
            ),
        ),
    ]



from allianceauth import hooks
from allianceauth.services.hooks import MenuItemHook, UrlHook
from django.utils.translation import gettext_lazy as _

from . import urls


class OpcalendarMenuItem(MenuItemHook):
    """This class ensures only authorized users will see the menu entry"""

    def __init__(self):
        # setup menu entry for sidebar
        MenuItemHook.__init__(
            self,
            _("Operation Calendar"),
            "far fa-calendar-alt",
            "opcalendar:calendar",
            navactive=["opcalendar:calendar"],
        )

    def render(self, request):
        if request.user.has_perm("opcalendar.basic_access"):
            return MenuItemHook.render(self, request)
        return ""


@hooks.register("menu_item_hook")
def register_menu():
    return OpcalendarMenuItem()


@hooks.register("url_hook")
def register_urls():
    return UrlHook(urls, "opcalendar", r"^opcalendar/")


@hooks.register("discord_cogs_hook")
def register_cogs():
    return ["opcalendar.cogs.ops"]



# Generated by Django 3.1.2 on 2021-01-08 08:47

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("opcalendar", "0010_auto_20201105_1602"),
    ]

    operations = [
        migrations.CreateModel(
            name="EventImport",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "source",
                    models.CharField(
                        choices=[("Spectre Fleet", "Spectre Fleet")], max_length=32
                    ),
                ),
                (
                    "host",
                    models.ForeignKey(
                        default=1,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="opcalendar.eventhost",
                    ),
                ),
                (
                    "operation_type",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="opcalendar.eventcategory",
                    ),
                ),
            ],
            options={
                "verbose_name": "Event Import",
                "verbose_name_plural": "Event Imports",
            },
        ),
    ]



# cal/views.py
import calendar
from datetime import date, datetime, timedelta

from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo
from allianceauth.services.hooks import get_extension_logger
from dateutil.relativedelta import relativedelta
from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.contrib.auth.mixins import LoginRequiredMixin, PermissionRequiredMixin
from django.core import serializers
from django.db import Error, transaction
from django.db.models import Q
from django.http import HttpResponse, HttpResponseRedirect, JsonResponse
from django.shortcuts import get_object_or_404, redirect, render
from django.urls import reverse
from django.utils.html import format_html
from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy
from django.utils.translation import gettext_lazy as _
from django.views.generic import ListView
from django_ical.views import ICalFeed
from esi.decorators import token_required

from opcalendar.models import (
    Event,
    EventCategory,
    EventMember,
    EventVisibility,
    IngameEvents,
    Owner,
)

from . import tasks
from .app_settings import get_site_url, moonmining_active, structuretimers_active
from .calendar import Calendar
from .forms import EventEditForm, EventForm
from .utils import messages_plus

logger = get_extension_logger(__name__)


@login_required(login_url="signup")
def index(request):
    return HttpResponse("hello")


@token_required(scopes=["esi-calendar.read_calendar_events.v1"])
def add_ingame_calendar(request, token):
    token_char = EveCharacter.objects.get(character_id=token.character_id)

    success = True
    try:
        owned_char = CharacterOwnership.objects.get(
            user=request.user, character=token_char
        )
    except CharacterOwnership.DoesNotExist:
        success = False
        owned_char = None

        messages_plus.error(
            request,
            format_html(
                gettext_lazy(
                    "You can only use your main or alt characters "
                    "to add corporations. "
                    "However, character %s is neither. "
                )
                % format_html("<strong>{}</strong>", token_char.character_name)
            ),
        )

    if success:
        try:
            corporation = EveCorporationInfo.objects.get(
                corporation_id=token_char.corporation_id
            )
        except EveCorporationInfo.DoesNotExist:
            corporation = EveCorporationInfo.objects.create_corporation(
                token_char.corporation_id
            )

        with transaction.atomic():
            owner, _ = Owner.objects.update_or_create(
                corporation=corporation, defaults={"character": owned_char}
            )

            owner.save()

        tasks.update_events_for_owner(owner_pk=owner.pk)

        messages_plus.success(
            request,
            format_html(
                gettext_lazy(
                    "Succesfully added ingame calendar sync for %(character)s. Process will run on background. You will receive a report once the process is finished."
                )
                % {
                    "corporation": format_html("<strong>{}</strong>", owner),
                    "character": format_html(
                        "<strong>{}</strong>", owner.character.character.character_name
                    ),
                }
            ),
        )

    return redirect("opcalendar:calendar")


def get_date(req_day):
    if req_day:
        year, month = (int(x) for x in req_day.split("-"))
        return date(year, month, day=1)
    return datetime.today()


def prev_month(d):
    first = d.replace(day=1)
    prev_month = first - timedelta(days=1)
    month = "month=" + str(prev_month.year) + "-" + str(prev_month.month)
    return month


def next_month(d):
    days_in_month = calendar.monthrange(d.year, d.month)[1]
    last = d.replace(day=days_in_month)
    next_month = last + timedelta(days=1)
    month = "month=" + str(next_month.year) + "-" + str(next_month.month)
    return month


class CalendarView(LoginRequiredMixin, PermissionRequiredMixin, ListView):
    permission_required = "opcalendar.basic_access"
    login_url = "signup"
    model = Event
    template_name = "opcalendar/calendar.html"

    def get_context_data(self, **kwargs):
        user = self.request.user
        # user_permissions = self.request.user.has_perm('app.get_main_view')
        context = super().get_context_data(**kwargs)
        d = get_date(self.request.GET.get("month", None))
        cal = Calendar(d.year, d.month, user)
        html_cal = cal.formatmonth(withyear=True)
        context["moonmining_active"] = moonmining_active()
        context["structuretimers_active"] = structuretimers_active()
        context["category"] = EventCategory.objects.all()
        context["visibility"] = EventVisibility.objects.all()
        context["calendar"] = mark_safe(html_cal)
        context["prev_month"] = prev_month(d)
        context["next_month"] = next_month(d)
        return context


@login_required
@permission_required("opcalendar.create_event")
def create_event(request):
    form = EventForm(request.POST or None)
    if request.POST and form.is_valid():
        event_count = 0

        # Get character
        character = request.user.profile.main_character
        operation_type = form.cleaned_data["operation_type"]
        title = form.cleaned_data["title"]
        host = form.cleaned_data["host"]
        doctrine = form.cleaned_data["doctrine"]
        formup_system = form.cleaned_data["formup_system"]
        description = form.cleaned_data["description"]
        start_time = form.cleaned_data["start_time"]
        end_time = form.cleaned_data["end_time"]
        repeat_event = form.cleaned_data["repeat_event"]
        repeat_times = form.cleaned_data["repeat_times"]
        fc = form.cleaned_data["fc"]
        event_visibility = form.cleaned_data["event_visibility"]

        # Add original event to objects list
        event = Event(
            user=request.user,
            operation_type=operation_type,
            title=title,
            host=host,
            doctrine=doctrine,
            formup_system=formup_system,
            description=description,
            start_time=start_time,
            end_time=end_time,
            repeat_event=repeat_event,
            repeat_times=repeat_times,
            eve_character=character,
            fc=fc,
            event_visibility=event_visibility,
        )

        try:
            event.save()
        except Error as e:
            logger.error("Error creating event %s: %s" % (event, e))

        # If we have a repeating event add event to object list multiple times
        if repeat_event:
            logger.debug("Event repeat %s for %s times" % (repeat_event, repeat_times))
            for repeat in range(repeat_times):
                if repeat_event == "DD":
                    start_time += relativedelta(days=1)
                    end_time += relativedelta(days=1)
                if repeat_event == "WE":
                    start_time += relativedelta(weeks=1)
                    end_time += relativedelta(weeks=1)
                if repeat_event == "MM":
                    start_time += relativedelta(months=1)
                    end_time += relativedelta(months=1)
                if repeat_event == "YY":
                    start_time += relativedelta(years=1)
                    end_time += relativedelta(years=1)

                event = Event(
                    user=request.user,
                    operation_type=operation_type,
                    title=title,
                    host=host,
                    doctrine=doctrine,
                    formup_system=formup_system,
                    description=description,
                    start_time=start_time,
                    end_time=end_time,
                    repeat_event=repeat_event,
                    repeat_times=repeat_times,
                    eve_character=character,
                    fc=fc,
                    event_visibility=event_visibility,
                )

                event_count += 1

                try:
                    event.save()
                except Error as e:
                    logger.error("Error creating event %s: %s" % (event, e))

        if event_count == 0:
            messages.success(
                request,
                ("Event %(opname)s created for %(date)s.")
                % {"opname": title, "date": start_time.strftime("%Y-%m-%d %H:%M")},
            )
        else:
            messages.success(
                request,
                (
                    "Event %(opname)s created for %(date)s. %(event_count)s duplicated events created."
                )
                % {
                    "opname": title,
                    "date": start_time.strftime("%Y-%m-%d %H:%M"),
                    "event_count": event_count,
                },
            )

        return HttpResponseRedirect(reverse("opcalendar:calendar"))

    return render(request, "opcalendar/event-add.html", {"form": form})


def get_category(request):
    catecoty_id = request.GET.get("category", None)

    data = {
        "category": serializers.serialize(
            "json", EventCategory.objects.all().filter(id=catecoty_id)
        )
    }
    return JsonResponse(data)


@login_required
@permission_required("opcalendar.basic_access")
def event_details(request, event_id):
    try:
        event = (
            Event.objects.filter(
                Q(event_visibility__restricted_to_group__in=request.user.groups.all())
                | Q(event_visibility__restricted_to_group__isnull=True),
            )
            .filter(
                Q(event_visibility__restricted_to_state=request.user.profile.state)
                | Q(event_visibility__restricted_to_state__isnull=True),
            )
            .get(id=event_id)
        )
        eventmember = EventMember.objects.filter(event=event)
        memberlist = []
        for member in eventmember:
            memberlist.append(member.character.character_name)

        context = {"event": event, "eventmember": eventmember, "memberlist": memberlist}

        return render(request, "opcalendar/event-details.html", context)

    except Event.DoesNotExist:
        return redirect("opcalendar:calendar")


@login_required
@permission_required("opcalendar.create_event")
def EventEdit(request, event_id):
    logger.debug(
        "edit_event called by user %s for optimer id %s" % (request.user, event_id)
    )
    event = get_object_or_404(Event, id=event_id)

    if request.method == "POST":
        form = EventEditForm(request.POST)
        logger.debug(
            "Received POST request containing update optimer form, is valid: %s"
            % form.is_valid()
        )
        if form.is_valid():
            form = EventEditForm(request.POST, instance=event)

            form.save()

            logger.info("User %s updating optimer id %s " % (request.user, event_id))

            messages.success(
                request,
                _("Saved changes to event for %(event)s.") % {"event": event.title},
            )
            url = reverse("opcalendar:event-detail", kwargs={"event_id": event.id})
            return HttpResponseRedirect(url)
    else:
        form = EventEditForm(instance=event)

    return render(request, "opcalendar/event-edit.html", context={"form": form})


@login_required
@permission_required("opcalendar.basic_access")
def ingame_event_details(request, event_id):
    event = IngameEvents.objects.get(event_id=event_id)

    context = {"event": event}

    if request.user.has_perm("opcalendar.view_ingame"):
        return render(request, "opcalendar/ingame-event-details.html", context)
    else:
        return redirect("opcalendar:calendar")


@login_required
@permission_required("opcalendar.create_event")
def EventDeleteView(request, event_id):
    logger.debug(
        "remove_optimer called by user %s for operation id %s"
        % (request.user, event_id)
    )
    op = get_object_or_404(Event, id=event_id)
    op.delete()
    logger.info("Deleting optimer id %s by user %s" % (event_id, request.user))
    messages.error(request, _("Removed event %(opname)s.") % {"opname": op.title})
    return redirect("opcalendar:calendar")


@login_required
@permission_required("opcalendar.basic_access")
def EventMemberSignup(request, event_id):
    event = Event.objects.get(id=event_id)

    character = request.user.profile.main_character

    EventMember.objects.update_or_create(event=event, character=character)

    messages.success(
        request,
        _("Succesfully signed up for event: %(event)s with %(character)s.")
        % {"event": event, "character": character},
    )

    return HttpResponseRedirect(request.META.get("HTTP_REFERER"))


@login_required
@permission_required("opcalendar.basic_access")
def EventMemberRemove(request, event_id):
    event = Event.objects.get(id=event_id)

    character = request.user.profile.main_character

    eventmember = EventMember.objects.filter(event=event, character=character)

    eventmember.delete()

    messages.error(
        request,
        _("Removed signup for event: %(event)s for %(character)s.")
        % {"event": event, "character": character},
    )

    return HttpResponseRedirect(request.META.get("HTTP_REFERER"))


class EventFeed(ICalFeed):
    """
    A simple event calender
    """

    product_id = "-//{}//Opcalendar//FEED".format(get_site_url())
    timezone = "UTC"
    file_name = "event.ics"

    def items(self):
        return (
            Event.objects.all()
            .order_by("-start_time")
            .filter(event_visibility__include_in_feed=True)
        )

    def item_guid(self, item):
        return "{}{}".format(item.id, "global_name")

    def item_title(self, item):
        return "{}".format(item.title)

    def item_description(self, item):
        return item.description

    def item_class(self, item):
        return item.title

    def item_location(self, item):
        return item.formup_system

    def item_start_datetime(self, item):
        return item.start_time

    def item_end_datetime(self, item):
        return item.end_time

    def item_link(self, item):
        return "{0}/opcalendar/event/{1}/details/".format(get_site_url(), item.id)


class EventIcalView(ICalFeed):
    """
    A simple event calender
    """

    product_id = "-//{}//Opcalendar//FEED".format(get_site_url())
    timezone = "UTC"
    file_name = "event.ics"

    def __call__(self, request, event_id, *args, **kwargs):
        self.request = request
        self.event_id = event_id
        return super(EventIcalView, self).__call__(request, event_id, *args, **kwargs)

    def items(self, event_id):
        return (
            Event.objects.all()
            .filter(id=self.event_id)
            .filter(
                Q(
                    event_visibility__restricted_to_group__in=self.request.user.groups.all()
                )
                | Q(event_visibility__restricted_to_group__isnull=True),
            )
            .filter(
                Q(event_visibility__restricted_to_state=self.request.user.profile.state)
                | Q(event_visibility__restricted_to_state__isnull=True),
            )
        )

    def item_guid(self, item):
        return "{}{}".format(item.id, "global_name")

    def item_title(self, item):
        return "{}".format(item.title)

    def item_description(self, item):
        return item.description

    def item_class(self, item):
        return item.title

    def item_location(self, item):
        return item.formup_system

    def item_start_datetime(self, item):
        return item.start_time

    def item_end_datetime(self, item):
        return item.end_time

    def item_organizer(self, item):
        return item.host

    def item_link(self, item):
        return "{0}/opcalendar/event/{1}/details/".format(get_site_url(), item.id)



import datetime as dt
from unittest.mock import patch

import requests
import requests_mock
from allianceauth.tests.auth_utils import AuthUtils
from pytz import utc

from .. import tasks
from ..models import Event, EventCategory, EventHost, EventImport
from ..utils import NoSocketsTestCase
from .testdata import COLOR_PURPLE, feedparser_parse, generate_ical_string

MODULE_PATH = "opcalendar.tasks"


@patch(MODULE_PATH + ".feedparser")
@requests_mock.Mocker()
class TestImportNpsiFleet(NoSocketsTestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.user = AuthUtils.create_user("Bruce Wayne")
        cls.eve_character = AuthUtils.add_main_character_2(
            cls.user, "Bruce Wayne", 1001, 2001
        )
        cls.host = EventHost.objects.create(community="Test Host")
        cls.category = EventCategory.objects.create(
            name="NPSI", ticker="NPSI", color=COLOR_PURPLE
        )

    ########################
    # spectre fleets only

    def test_should_add_new_spectre_fleet_event(self, mock_feedparser, requests_mocker):
        # given
        mock_feedparser.parse = feedparser_parse
        EventImport.objects.create(
            source=EventImport.SPECTRE_FLEET,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
            eve_character=self.eve_character,
        )
        # when
        tasks.import_all_npsi_fleets()
        # then
        self.assertEqual(Event.objects.count(), 1)
        obj = Event.objects.first()
        self.assertEqual(obj.operation_type, self.category)
        self.assertEqual(obj.title, "Spectre Fleet 1")
        self.assertEqual(obj.host, self.host)
        self.assertEqual(obj.doctrine, "see details")
        self.assertEqual(obj.formup_system, EventImport.SPECTRE_FLEET)
        self.assertEqual(obj.description, "")
        published = utc.localize(dt.datetime(2021, 2, 5, 21, 0))
        self.assertEqual(obj.start_time, published)
        self.assertEqual(obj.end_time, published)
        self.assertEqual(obj.fc, EventImport.SPECTRE_FLEET)
        self.assertEqual(obj.user, self.user)
        self.assertEqual(obj.eve_character, self.eve_character)

    def test_should_add_new_spectre_fleet_event_no_character(
        self, mock_feedparser, requests_mocker
    ):
        # given
        mock_feedparser.parse = feedparser_parse
        EventImport.objects.create(
            source=EventImport.SPECTRE_FLEET,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
        )
        # when
        tasks.import_all_npsi_fleets()
        # then
        self.assertTrue(Event.objects.filter(title="Spectre Fleet 1").exists())

    def test_should_not_replace_existing_spectre_fleet_event(
        self, mock_feedparser, requests_mocker
    ):
        # given
        mock_feedparser.parse = feedparser_parse
        EventImport.objects.create(
            source=EventImport.SPECTRE_FLEET,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
            eve_character=self.eve_character,
        )
        published = utc.localize(dt.datetime(2021, 2, 5, 21, 0))
        original_event = Event.objects.create(
            operation_type=self.category,
            title="Spectre Fleet 1",
            host=self.host,
            doctrine="see details",
            formup_system=EventImport.SPECTRE_FLEET,
            description="Testing Eve Uni class 1",
            start_time=published,
            end_time=published,
            fc=EventImport.SPECTRE_FLEET,
            external=True,
            user=self.user,
            eve_character=self.eve_character,
        )
        # when
        tasks.import_all_npsi_fleets()
        # then
        self.assertEqual(Event.objects.count(), 1)
        self.assertTrue(Event.objects.filter(pk=original_event.pk).exists())

    def test_should_delete_outdated_spectre_fleet_event(
        self, mock_feedparser, requests_mocker
    ):
        # given
        mock_feedparser.parse = lambda x: feedparser_parse("no-data")
        EventImport.objects.create(
            source=EventImport.SPECTRE_FLEET,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
            eve_character=self.eve_character,
        )
        published = utc.localize(dt.datetime(2021, 2, 3, 21, 0))
        Event.objects.create(
            operation_type=self.category,
            title="Spectre Fleet OLD",
            host=self.host,
            doctrine="see details",
            formup_system=EventImport.SPECTRE_FLEET,
            description="Testing Eve Uni class OLD",
            start_time=published,
            end_time=published,
            fc=EventImport.SPECTRE_FLEET,
            external=True,
            user=self.user,
            eve_character=self.eve_character,
        )
        # when
        tasks.import_all_npsi_fleets()
        # then
        self.assertEqual(Event.objects.count(), 0)

    def test_should_report_when_spectre_fleet_has_error(
        self, mock_feedparser, requests_mocker
    ):
        # given
        mock_feedparser.parse.side_effect = RuntimeError
        EventImport.objects.create(
            source=EventImport.SPECTRE_FLEET,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
            eve_character=self.eve_character,
        )
        # when
        result = tasks.import_all_npsi_fleets()
        # then
        self.assertFalse(result)
        self.assertEqual(Event.objects.count(), 0)

    ########################
    # fun inc only

    def test_should_add_new_fun_inc_event(self, mock_feedparser, requests_mocker):
        # given
        requests_mocker.register_uri(
            "GET",
            url="https://calendar.google.com/calendar/ical/og3uh76l8ul3dfgbie03fbbgs8%40group.calendar.google.com/private-f466889b44741fd7249e99e21ac171ff/basic.ics",
            text=generate_ical_string("fun_inc"),
        )
        EventImport.objects.create(
            source=EventImport.FUN_INC,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
            eve_character=self.eve_character,
        )
        # when
        tasks.import_all_npsi_fleets()
        # then
        self.assertEqual(Event.objects.count(), 1)
        obj = Event.objects.first()
        self.assertEqual(obj.operation_type, self.category)
        self.assertEqual(obj.title, "Fun Fleet 1")
        self.assertEqual(obj.host, self.host)
        self.assertEqual(obj.doctrine, "see details")
        self.assertEqual(obj.formup_system, EventImport.FUN_INC)
        self.assertEqual(obj.description, "Testing Fun Fleet 1")
        self.assertEqual(obj.start_time, utc.localize(dt.datetime(2021, 2, 5, 22, 0)))
        self.assertEqual(obj.end_time, utc.localize(dt.datetime(2021, 2, 5, 23, 0)))
        self.assertEqual(obj.fc, EventImport.FUN_INC)
        self.assertEqual(obj.user, self.user)
        self.assertEqual(obj.eve_character, self.eve_character)

    def test_should_add_new_fun_inc_event_no_character(
        self, mock_feedparser, requests_mocker
    ):
        # given
        requests_mocker.register_uri(
            "GET",
            url="https://calendar.google.com/calendar/ical/og3uh76l8ul3dfgbie03fbbgs8%40group.calendar.google.com/private-f466889b44741fd7249e99e21ac171ff/basic.ics",
            text=generate_ical_string("fun_inc"),
        )
        EventImport.objects.create(
            source=EventImport.FUN_INC,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
        )
        # when
        tasks.import_all_npsi_fleets()
        # then
        self.assertTrue(Event.objects.filter(title="Fun Fleet 1").exists())

    def test_should_not_replace_existing_fun_inc_event(
        self, mock_feedparser, requests_mocker
    ):
        # given
        requests_mocker.register_uri(
            "GET",
            url="https://calendar.google.com/calendar/ical/og3uh76l8ul3dfgbie03fbbgs8%40group.calendar.google.com/private-f466889b44741fd7249e99e21ac171ff/basic.ics",
            text=generate_ical_string("fun_inc"),
        )
        EventImport.objects.create(
            source=EventImport.FUN_INC,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
            eve_character=self.eve_character,
        )
        original_event = Event.objects.create(
            operation_type=self.category,
            title="Fun Fleet 1",
            host=self.host,
            doctrine="see details",
            formup_system=EventImport.FUN_INC,
            description="Testing Fun Fleet 1",
            start_time=utc.localize(dt.datetime(2021, 2, 5, 22, 0)),
            end_time=utc.localize(dt.datetime(2021, 2, 5, 23, 0)),
            fc=EventImport.FUN_INC,
            external=True,
            user=self.user,
            eve_character=self.eve_character,
        )
        # when
        tasks.import_all_npsi_fleets()
        # then
        self.assertEqual(Event.objects.count(), 1)
        self.assertTrue(Event.objects.filter(pk=original_event.pk).exists())

    def test_should_delete_outdated_fun_inc_event(
        self, mock_feedparser, requests_mocker
    ):
        # given
        requests_mocker.register_uri(
            "GET",
            url="https://calendar.google.com/calendar/ical/og3uh76l8ul3dfgbie03fbbgs8%40group.calendar.google.com/private-f466889b44741fd7249e99e21ac171ff/basic.ics",
            text=generate_ical_string("no-data"),
        )
        EventImport.objects.create(
            source=EventImport.FUN_INC,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
            eve_character=self.eve_character,
        )
        Event.objects.create(
            operation_type=self.category,
            title="Fun Fleet OLD",
            host=self.host,
            doctrine="see details",
            formup_system=EventImport.FUN_INC,
            description="Testing Fun Fleet OLD",
            start_time=utc.localize(dt.datetime(2021, 2, 4, 22, 0)),
            end_time=utc.localize(dt.datetime(2021, 2, 4, 23, 0)),
            fc=EventImport.FUN_INC,
            external=True,
            user=self.user,
            eve_character=self.eve_character,
        )
        # when
        tasks.import_all_npsi_fleets()
        # then
        self.assertEqual(Event.objects.count(), 0)

    def test_should_report_when_fun_inc_calendar_is_invalid(
        self, mock_feedparser, requests_mocker
    ):
        # given
        requests_mocker.register_uri(
            "GET",
            url="https://calendar.google.com/calendar/ical/og3uh76l8ul3dfgbie03fbbgs8%40group.calendar.google.com/private-f466889b44741fd7249e99e21ac171ff/basic.ics",
            text="",
        )
        EventImport.objects.create(
            source=EventImport.FUN_INC,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
            eve_character=self.eve_character,
        )
        # when
        result = tasks.import_all_npsi_fleets()
        # then
        self.assertFalse(result)
        self.assertEqual(Event.objects.count(), 0)

    def test_should_report_when_fun_inc_calendar_request_has_error(
        self, mock_feedparser, requests_mocker
    ):
        # given
        requests_mocker.register_uri(
            "GET",
            url="https://calendar.google.com/calendar/ical/og3uh76l8ul3dfgbie03fbbgs8%40group.calendar.google.com/private-f466889b44741fd7249e99e21ac171ff/basic.ics",
            exc=requests.exceptions.ConnectTimeout,
        )
        EventImport.objects.create(
            source=EventImport.FUN_INC,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
            eve_character=self.eve_character,
        )
        # when
        result = tasks.import_all_npsi_fleets()
        # then
        self.assertFalse(result)
        self.assertEqual(Event.objects.count(), 0)

    ########################
    # eve uni only

    def test_should_add_new_eve_uni_event(self, mock_feedparser, requests_mocker):
        # given
        requests_mocker.register_uri(
            "GET",
            url="https://portal.eveuniversity.org/api/getcalendar",
            text=generate_ical_string("eve_uni"),
        )
        EventImport.objects.create(
            source=EventImport.EVE_UNIVERSITY,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
            eve_character=self.eve_character,
        )
        # when
        tasks.import_all_npsi_fleets()
        # then
        self.assertEqual(Event.objects.count(), 1)
        obj = Event.objects.first()
        self.assertEqual(obj.operation_type, self.category)
        self.assertEqual(obj.title, "Eve Uni class 1")
        self.assertEqual(obj.host, self.host)
        self.assertEqual(obj.doctrine, "see details")
        self.assertEqual(obj.formup_system, EventImport.EVE_UNIVERSITY)
        self.assertEqual(obj.description, "Testing Eve Uni class 1")
        self.assertEqual(obj.start_time, utc.localize(dt.datetime(2021, 2, 4, 22, 0)))
        self.assertEqual(obj.end_time, utc.localize(dt.datetime(2021, 2, 4, 23, 0)))
        self.assertEqual(obj.fc, EventImport.EVE_UNIVERSITY)
        self.assertEqual(obj.user, self.user)
        self.assertEqual(obj.eve_character, self.eve_character)

    def test_should_add_new_eve_uni_event_no_character(
        self, mock_feedparser, requests_mocker
    ):
        # given
        requests_mocker.register_uri(
            "GET",
            url="https://portal.eveuniversity.org/api/getcalendar",
            text=generate_ical_string("eve_uni"),
        )
        EventImport.objects.create(
            source=EventImport.EVE_UNIVERSITY,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
        )
        # when
        tasks.import_all_npsi_fleets()
        # then
        self.assertTrue(Event.objects.filter(title="Eve Uni class 1").exists())

    def test_should_not_replace_existing_eve_uni_event(
        self, mock_feedparser, requests_mocker
    ):
        # given
        requests_mocker.register_uri(
            "GET",
            url="https://portal.eveuniversity.org/api/getcalendar",
            text=generate_ical_string("eve_uni"),
        )
        EventImport.objects.create(
            source=EventImport.EVE_UNIVERSITY,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
            eve_character=self.eve_character,
        )
        original_event = Event.objects.create(
            operation_type=self.category,
            title="Eve Uni class 1",
            host=self.host,
            doctrine="see details",
            formup_system=EventImport.EVE_UNIVERSITY,
            description="Testing Eve Uni class 1",
            start_time=utc.localize(dt.datetime(2021, 2, 4, 22, 0)),
            end_time=utc.localize(dt.datetime(2021, 2, 4, 23, 0)),
            fc=EventImport.EVE_UNIVERSITY,
            external=True,
            user=self.user,
            eve_character=self.eve_character,
        )
        # when
        tasks.import_all_npsi_fleets()
        # then
        self.assertEqual(Event.objects.count(), 1)
        self.assertTrue(Event.objects.filter(pk=original_event.pk).exists())

    def test_should_delete_outdated_eve_uni_events(
        self, mock_feedparser, requests_mocker
    ):
        # given
        requests_mocker.register_uri(
            "GET",
            url="https://portal.eveuniversity.org/api/getcalendar",
            text=generate_ical_string("no-data"),
        )
        EventImport.objects.create(
            source=EventImport.EVE_UNIVERSITY,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
            eve_character=self.eve_character,
        )
        Event.objects.create(
            operation_type=self.category,
            title="Eve Uni class OLD",
            host=self.host,
            doctrine="see details",
            formup_system=EventImport.EVE_UNIVERSITY,
            description="Testing Eve Uni class OLD",
            start_time=utc.localize(dt.datetime(2021, 2, 3, 22, 0)),
            end_time=utc.localize(dt.datetime(2021, 2, 3, 23, 0)),
            fc=EventImport.EVE_UNIVERSITY,
            external=True,
            user=self.user,
            eve_character=self.eve_character,
        )
        # when
        tasks.import_all_npsi_fleets()
        # then
        self.assertEqual(Event.objects.count(), 0)

    def test_should_report_when_eve_uni_request_has_error(
        self, mock_feedparser, requests_mocker
    ):
        # given
        requests_mocker.register_uri(
            "GET",
            url="https://portal.eveuniversity.org/api/getcalendar",
            exc=requests.exceptions.ConnectTimeout,
        )
        EventImport.objects.create(
            source=EventImport.EVE_UNIVERSITY,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
            eve_character=self.eve_character,
        )
        # when
        result = tasks.import_all_npsi_fleets()
        # then
        self.assertFalse(result)
        self.assertEqual(Event.objects.count(), 0)

    def test_should_report_when_eve_uni_calendar_is_invalid(
        self, mock_feedparser, requests_mocker
    ):
        # given
        requests_mocker.register_uri(
            "GET", url="https://portal.eveuniversity.org/api/getcalendar", text=""
        )
        EventImport.objects.create(
            source=EventImport.EVE_UNIVERSITY,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
            eve_character=self.eve_character,
        )
        # when
        result = tasks.import_all_npsi_fleets()
        # then
        self.assertFalse(result)
        self.assertEqual(Event.objects.count(), 0)

    ########################
    # multiple fleet types

    def test_should_add_fleet_events_all_types(self, mock_feedparser, requests_mocker):
        # given
        mock_feedparser.parse = feedparser_parse
        EventImport.objects.create(
            source=EventImport.SPECTRE_FLEET,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
            eve_character=self.eve_character,
        )
        requests_mocker.register_uri(
            "GET",
            url="https://calendar.google.com/calendar/ical/og3uh76l8ul3dfgbie03fbbgs8%40group.calendar.google.com/private-f466889b44741fd7249e99e21ac171ff/basic.ics",
            text=generate_ical_string("fun_inc"),
        )
        EventImport.objects.create(
            source=EventImport.FUN_INC,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
            eve_character=self.eve_character,
        )
        requests_mocker.register_uri(
            "GET",
            url="https://portal.eveuniversity.org/api/getcalendar",
            text=generate_ical_string("eve_uni"),
        )
        EventImport.objects.create(
            source=EventImport.EVE_UNIVERSITY,
            host=self.host,
            operation_type=self.category,
            creator=self.user,
            eve_character=self.eve_character,
        )
        # when
        tasks.import_all_npsi_fleets()
        # then
        self.assertEqual(Event.objects.count(), 3)
        self.assertTrue(Event.objects.filter(title="Spectre Fleet 1").exists())
        self.assertTrue(Event.objects.filter(title="Fun Fleet 1").exists())
        self.assertTrue(Event.objects.filter(title="Eve Uni class 1").exists())



from django.contrib.sessions.models import Session
from django.utils import timezone
from django.contrib.auth.models import User


def get_current_user():
    active_sessions = Session.objects.filter(expire_date__gte=timezone.now())
    user_id_list = []
    for session in active_sessions:
        data = session.get_decoded()
        user_id_list.append(data.get("_auth_user_id", None))
    user = User.objects.get(id=user_id_list[0])
    return user



# Generated by Django 3.1.7 on 2021-03-30 06:32

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):
    dependencies = [
        ("authentication", "0017_remove_fleetup_permission"),
        ("auth", "0012_alter_user_first_name_max_length"),
        ("eveonline", "0014_auto_20210105_1413"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("opcalendar", "0022_auto_20210222_1255"),
    ]

    operations = [
        migrations.CreateModel(
            name="EventVisibility",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "name",
                    models.CharField(
                        help_text="Name for the visibility filter", max_length=150
                    ),
                ),
                (
                    "ignore_past_fleets",
                    models.BooleanField(
                        default=True,
                        help_text="Should we ignore fleet signals that are in the past",
                    ),
                ),
                (
                    "color",
                    models.CharField(
                        blank=True,
                        default="",
                        help_text="Color to be displayed on calendar",
                        max_length=7,
                    ),
                ),
                (
                    "is_active",
                    models.BooleanField(
                        default=True,
                        help_text="Whether this visibility filter is active",
                    ),
                ),
                (
                    "restricted_to_group",
                    models.ManyToManyField(
                        blank=True,
                        help_text="The group(s) that will be able to see this event visibility type ...",
                        related_name="eventvisibility_require_groups",
                        to="auth.Group",
                    ),
                ),
                (
                    "restricted_to_state",
                    models.ManyToManyField(
                        blank=True,
                        help_text="The state(s) that will be able to see this event visibility type ...",
                        related_name="eventvisibility_require_states",
                        to="authentication.State",
                    ),
                ),
            ],
            options={
                "verbose_name": "Event Visibility Filter",
                "verbose_name_plural": "Event Visibilities Filters",
            },
        ),
        migrations.AlterModelOptions(
            name="general",
            options={
                "default_permissions": (),
                "managed": False,
                "permissions": (
                    (
                        "basic_access",
                        "Can access this app and see operations based on visibility rules",
                    ),
                    ("create_event", "Can create and edit events"),
                    ("see_signups", "Can see all signups for event"),
                    ("manage_event", "Can delete and manage other signups"),
                    (
                        "add_ingame_calendar_owner",
                        "Can add ingame calendar feeds for their corporation",
                    ),
                ),
            },
        ),
        migrations.AddField(
            model_name="event",
            name="external",
            field=models.BooleanField(
                default=False,
                help_text="Is the event an external event over API",
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="external",
            field=models.BooleanField(
                default=False,
                help_text="External hosts are for NPSI API imports. Checking this box will hide the host in the manual event form.",
            ),
        ),
        migrations.AddField(
            model_name="eventmember",
            name="character",
            field=models.ForeignKey(
                help_text="Event creator main character",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="eveonline.evecharacter",
            ),
        ),
        migrations.AddField(
            model_name="ingameevents",
            name="event_owner_id",
            field=models.IntegerField(null=True),
        ),
        migrations.AddField(
            model_name="ingameevents",
            name="host",
            field=models.ForeignKey(
                default=1,
                help_text="Host entity for the event",
                on_delete=django.db.models.deletion.CASCADE,
                to="opcalendar.eventhost",
            ),
        ),
        migrations.AddField(
            model_name="ingameevents",
            name="moon_extraction",
            field=models.BooleanField(default=False, null=True),
        ),
        migrations.AddField(
            model_name="owner",
            name="operation_type",
            field=models.ForeignKey(
                blank=True,
                help_text="Event category that will be assigned for all of the events from this owner.",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="opcalendar.eventcategory",
            ),
        ),
        migrations.AlterField(
            model_name="event",
            name="created_date",
            field=models.DateTimeField(
                default=django.utils.timezone.now,
                help_text="When the event was created",
            ),
        ),
        migrations.AlterField(
            model_name="event",
            name="description",
            field=models.TextField(help_text="Description text for the operation"),
        ),
        migrations.AlterField(
            model_name="event",
            name="doctrine",
            field=models.CharField(help_text="Doctrine URL or name", max_length=254),
        ),
        migrations.AlterField(
            model_name="event",
            name="end_time",
            field=models.DateTimeField(help_text="Event end date and time"),
        ),
        migrations.AlterField(
            model_name="event",
            name="eve_character",
            field=models.ForeignKey(
                help_text="Character used to create the event",
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="eveonline.evecharacter",
            ),
        ),
        migrations.AlterField(
            model_name="event",
            name="fc",
            field=models.CharField(
                help_text="Fleet commander/manager for the event", max_length=254
            ),
        ),
        migrations.AlterField(
            model_name="event",
            name="formup_system",
            field=models.CharField(help_text="Location for formup", max_length=254),
        ),
        migrations.AlterField(
            model_name="event",
            name="host",
            field=models.ForeignKey(
                help_text="Host entity for the event",
                on_delete=django.db.models.deletion.CASCADE,
                to="opcalendar.eventhost",
            ),
        ),
        migrations.AlterField(
            model_name="event",
            name="operation_type",
            field=models.ForeignKey(
                help_text="Event category type",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="opcalendar.eventcategory",
            ),
        ),
        migrations.AlterField(
            model_name="event",
            name="start_time",
            field=models.DateTimeField(help_text="Event start date and time"),
        ),
        migrations.AlterField(
            model_name="event",
            name="title",
            field=models.CharField(help_text="Title for the event", max_length=200),
        ),
        migrations.AlterField(
            model_name="event",
            name="user",
            field=models.ForeignKey(
                help_text="User who created the event",
                on_delete=django.db.models.deletion.CASCADE,
                to=settings.AUTH_USER_MODEL,
            ),
        ),
        migrations.AlterField(
            model_name="eventcategory",
            name="color",
            field=models.CharField(
                blank=True,
                default="",
                help_text="Color to be displayed on calendar",
                max_length=7,
            ),
        ),
        migrations.AlterField(
            model_name="eventcategory",
            name="name",
            field=models.CharField(help_text="Name for the category", max_length=150),
        ),
        migrations.AlterField(
            model_name="eventcategory",
            name="ticker",
            field=models.CharField(help_text="Ticker for the category", max_length=10),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="ingame_mailing_list",
            field=models.CharField(
                blank=True, help_text="Ingame mailing list name", max_length=150
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="logo_url",
            field=models.CharField(
                blank=True,
                help_text="Absolute URL for the community logo",
                max_length=256,
            ),
        ),
        migrations.AlterField(
            model_name="eventhost",
            name="website",
            field=models.CharField(
                blank=True, help_text="Website link URL", max_length=150
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="enabled",
            field=models.BooleanField(
                default=True, help_text="Is the webhook enabled?"
            ),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="name",
            field=models.CharField(help_text="Name for this webhook", max_length=150),
        ),
        migrations.AlterField(
            model_name="webhook",
            name="webhook_url",
            field=models.CharField(help_text="Webhook URL", max_length=500),
        ),
        migrations.AlterUniqueTogether(
            name="event",
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name="eventmember",
            unique_together={("event", "character")},
        ),
        migrations.DeleteModel(
            name="EventSignal",
        ),
        migrations.AddField(
            model_name="eventvisibility",
            name="webhook",
            field=models.ForeignKey(
                blank=True,
                help_text="Webhook to send over notifications about these fleet types",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="opcalendar.webhook",
            ),
        ),
        migrations.RemoveField(
            model_name="event",
            name="visibility",
        ),
        migrations.RemoveField(
            model_name="eventmember",
            name="user",
        ),
        migrations.AddField(
            model_name="event",
            name="event_visibility",
            field=models.ForeignKey(
                blank=True,
                help_text="Visibility filter that dictates who is able to see this event",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="opcalendar.eventvisibility",
            ),
        ),
        migrations.AddField(
            model_name="eventimport",
            name="event_visibility",
            field=models.ForeignKey(
                help_text="Visibility filter that dictates who is able to see this event",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="opcalendar.eventvisibility",
            ),
        ),
        migrations.AddField(
            model_name="owner",
            name="event_visibility",
            field=models.ForeignKey(
                blank=True,
                help_text="Visibility filter that dictates who is able to see this event",
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                to="opcalendar.eventvisibility",
            ),
        ),
    ]



# Generated by Django 3.1.7 on 2021-04-29 11:11

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("opcalendar", "0023_auto_20210330_0632"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="ingameevents",
            name="moon_extraction",
        ),
        migrations.AddField(
            model_name="eventcategory",
            name="description",
            field=models.TextField(
                blank=True,
                help_text="Prefilled description that will be added on default on the event description.",
            ),
        ),
        migrations.AddField(
            model_name="eventhost",
            name="is_default",
            field=models.BooleanField(
                default=False,
                help_text="Whether this host is used as the default value on the event form",
            ),
        ),
        migrations.AddField(
            model_name="eventvisibility",
            name="include_in_feed",
            field=models.BooleanField(
                default=False,
                help_text="Whether these events should be included in the ical feed.",
            ),
        ),
        migrations.AddField(
            model_name="eventvisibility",
            name="is_default",
            field=models.BooleanField(
                default=False,
                help_text="Whether this visibility filter is used as the default value on the event form",
            ),
        ),
        migrations.AddField(
            model_name="eventvisibility",
            name="is_visible",
            field=models.BooleanField(
                default=True,
                help_text="Whether this visibility filter should be displayed on the event form. Disable for internal visibilities such as the NPSI import fleet visibilities.",
            ),
        ),
        migrations.AlterField(
            model_name="eventimport",
            name="source",
            field=models.CharField(
                choices=[
                    ("LN", "EVE LinkNet"),
                    ("SF", "Spectre Fleet"),
                    ("EU", "EVE University"),
                    ("FI", "Fun Inc."),
                    ("FY", "FRIDAY YARRRR"),
                    ("RR", "Redemption Road"),
                    ("CA", "CAS"),
                    ("FD", "Fwaming Dwagons"),
                    ("FR", "FREE RANGE CHIKUNS"),
                ],
                help_text="The API source where you want to pull events from",
                max_length=32,
            ),
        ),
    ]



import datetime as dt
from unittest.mock import patch

from django.db.models import signals
from allianceauth.tests.auth_utils import AuthUtils
from pytz import utc

from ..models import EventCategory, EventHost, IngameEvents, Owner, Event
from ..signals import fleet_deleted, fleet_saved
from ..utils import NoSocketsTestCase, add_character_to_user_2, add_new_token
from .testdata import (
    COLOR_PURPLE,
    esi_get_characters_character_id_calendar,
    esi_get_characters_character_id_calendar_event_id,
)

MODULE_PATH = "opcalendar.models"


@patch(MODULE_PATH + ".esi")
class TestOwnerUpdateEventsEsi(NoSocketsTestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        cls.user = AuthUtils.create_user("Bruce Wayne")
        cls.user = AuthUtils.add_permission_to_user_by_name(
            "opcalendar.add_ingame_calendar_owner", cls.user
        )
        cls.eve_character = add_character_to_user_2(
            cls.user, 1001, "Bruce Wayne", 2001, "Wayne Technologies"
        )
        add_new_token(
            cls.user, cls.eve_character, ["esi-calendar.read_calendar_events.v1"]
        )
        cls.host = EventHost.objects.create(community="Test Host")
        cls.category = EventCategory.objects.create(
            name="NPSI", ticker="NPSI", color=COLOR_PURPLE
        )
        cls.owner = Owner.objects.create(
            character=cls.user.character_ownerships.first()
        )

    def setUp(self) -> None:
        signals.post_save.disconnect(fleet_saved, sender=Event)
        signals.post_save.disconnect(fleet_saved, sender=IngameEvents)
        signals.pre_delete.disconnect(fleet_deleted, sender=Event)
        signals.pre_delete.disconnect(fleet_deleted, sender=IngameEvents)

    def tearDown(self) -> None:
        signals.pre_delete.connect(fleet_deleted, sender=Event)
        signals.pre_delete.connect(fleet_deleted, sender=IngameEvents)
        signals.post_save.connect(fleet_saved, sender=Event)
        signals.post_save.connect(fleet_saved, sender=IngameEvents)

    def test_should_add_new_events(self, mock_esi):
        # given
        mock_esi.client.Calendar.get_characters_character_id_calendar = (
            esi_get_characters_character_id_calendar
        )
        mock_esi.client.Calendar.get_characters_character_id_calendar_event_id = (
            esi_get_characters_character_id_calendar_event_id
        )
        # when
        self.owner.update_events_esi()
        # then
        self.assertEqual(IngameEvents.objects.count(), 1)
        obj = IngameEvents.objects.first()
        self.assertEqual(obj.event_id, 1386435)
        self.assertEqual(obj.owner, self.owner)
        self.assertEqual(obj.text, "The EVE Online Show features latest developer news")
        self.assertEqual(obj.owner_name, "EVE System")
        self.assertEqual(obj.owner_type, "eve_server")
        self.assertEqual(obj.importance, "1")
        self.assertEqual(obj.duration, "60")
        self.assertEqual(
            obj.event_start_date, utc.localize(dt.datetime(2016, 6, 26, 21, 0))
        )
        self.assertEqual(
            obj.event_end_date, utc.localize(dt.datetime(2016, 6, 26, 22, 0))
        )
        self.assertEqual(obj.title, "o7 The EVE Online Show")

    def test_should_not_replace_existing_events(self, mock_esi):
        # given
        mock_esi.client.Calendar.get_characters_character_id_calendar = (
            esi_get_characters_character_id_calendar
        )
        mock_esi.client.Calendar.get_characters_character_id_calendar_event_id = (
            esi_get_characters_character_id_calendar_event_id
        )
        original_event = IngameEvents.objects.create(
            event_id=1386435,
            event_start_date=utc.localize(dt.datetime(2016, 6, 26, 21, 0)),
            event_end_date=utc.localize(dt.datetime(2016, 6, 26, 22, 0)),
            owner=self.owner,
            text="The EVE Online Show features latest developer news",
            owner_type="eve_server",
            owner_name="EVE Server",
            importance="1",
            duration="30",
        )
        # when
        self.owner.update_events_esi()
        # then
        self.assertEqual(IngameEvents.objects.count(), 1)
        self.assertTrue(IngameEvents.objects.filter(pk=original_event.pk).exists())



from django.shortcuts import render, redirect
from django.contrib.auth import authenticate, login, logout
from opcalendar.forms import SignupForm


def signup(request):
    forms = SignupForm()
    if request.method == "POST":
        forms = SignupForm(request.POST)
        if forms.is_valid():
            username = forms.cleaned_data["username"]
            password = forms.cleaned_data["password"]
            user = authenticate(username=username, password=password)
            if user:
                login(request, user)
                return redirect("opcalendar:calendar")
    context = {"form": forms}
    return render(request, "signup.html", context)


def user_logout(request):
    logout(request)
    return redirect("signup")



# Generated by Django 3.1.2 on 2020-11-05 16:02

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("opcalendar", "0009_auto_20201105_0859"),
    ]

    operations = [
        migrations.AlterField(
            model_name="event",
            name="visibility",
            field=models.CharField(
                choices=[
                    ("public", "Public access"),
                    ("member", "Members only access"),
                ],
                db_index=True,
                default="public",
                max_length=7,
            ),
        ),
    ]
